
% Function X=genlikSI_fun(Xnoisy,NoiseStd,W,h,SigIntDEF)
% applies wavelet domain image denoisinig using generalized likelihood ratio. 
% Algorithm: GENLIK algorithm with the shift invariant (SI) transform and 
% using the generalized Laplacian prior, described in: 
% A. Pizurica, "Image Denoising Using Wavelets and Spatial Context Modeling",
% PhD. thesis, Ghent UNiversity, June 2002.
%
% INPUT:
%   Xnoisy - noisy image
%   NoiseStd - noise standard deviation
%   W - "half window size", e.g., W=1 for 3x3 window and W=2 for 5x5 window 
%   h - wavelet filter
%   SigIntDEF - A multiplication factor for adjusting the notion of the "signal of interest" 
%               Threshold that defines the signal of interest equals the noise standard deviation. 
%               This threshold yields the results that minimize the mean squred error. In some cases 
%               smaller threshold value may yield visually better result (more texture is preserved at 
%               the expense of the excessive noise). A user can define other threshold by choosing 
%               ``other signal of interest'' and specifying the value of the multiplication factor  
%               SigIntDEF for the threshold that define the signal of interest.
%
% OUTPUT:
%    X - denoised image
%
% 
% Author: Aleksandra Pizurica/TELIN/Ghent University
% E-mail: Aleksandra.Pizurica@telin.UGent.be
% Last modified: June 2003.
% Copyright: All software, documentation, and related files in this distribution
%            are Copyright (c) 2003 Aleksandra Pizurica. All rights reserved.
%
% This denoising routine uses wavelet decomposition and reconstruction functions
% mrdwt.c and mirdwt.c developed by Rice University, Houston, Texas and its contributors.
%
% This software is distributed and licensed to you on a non-exclusive 
% basis, free-of-charge. Redistribution and use in source and binary forms, 
% with or without modification, are permitted provided that the above 
% copyright notice is clearly maintained.


function X=genlikSI_fun(Xnoisy,NoiseStd,W,h,SigIntDEF);

[M,N]=size(Xnoisy); 


%=====================================================  
 % Decomposition 
  
 L=4;
 [yl,yh,L] = mrdwt(Xnoisy/4,h,L);
 % (input image is here rescaled to make faster the subsequent convolutions 
 % that apear in functions MagPDFs_varT.m and AverageMagPDFs.m) 
 
 LH1 = yh(:,1:N); 
 HL1 = yh(:,N+1:2*N); 
 HH1 = yh(:,2*N+1:3*N);
 
 LH2 = yh(:,3*N+1:4*N); 
 HL2 = yh(:,4*N+1:5*N); 
 HH2 = yh(:,5*N+1:6*N);
 
 LH3 = yh(:,6*N+1:7*N); 
 HL3 = yh(:,7*N+1:8*N); 
 HH3 = yh(:,8*N+1:9*N);
 
 LH4 = yh(:,9*N+1:10*N); 
 HL4 = yh(:,10*N+1:11*N); 
 HH4 = yh(:,11*N+1:12*N);
 
 
% Due to rescaling of gray values

sigma=NoiseStd/4;
T=SigIntDEF*sigma;

% Processing scale 2^4:
Scale=4;
HL4p=rem_noise_GenLap(HL4,W,sigma,T);
LH4p=rem_noise_GenLap(LH4,W,sigma,T);
HH4p=rem_noise_GenLap(HH4,W,sigma,T);


% Processing scale 2^3:
Scale=3;
HL3p=rem_noise_GenLap(HL3,W,sigma,T);
LH3p=rem_noise_GenLap(LH3,W,sigma,T);
HH3p=rem_noise_GenLap(HH3,W,sigma,T);

% Processing scale 2^2:
Scale=2;
HL2p=rem_noise_GenLap(HL2,W,sigma,T);
LH2p=rem_noise_GenLap(LH2,W,sigma,T);
HH2p=rem_noise_GenLap(HH2,W,sigma,T);

% Processing scale 2^1:
Scale=1;
HL1p=rem_noise_GenLap(HL1,W,sigma,T);
LH1p=rem_noise_GenLap(LH1,W,sigma,T);
HH1p=rem_noise_GenLap(HH1,W,sigma,T);


%================================================
% Reconstruction

yhp=yh;

 yhp(:,1:N)=LH1p; 
 yhp(:,N+1:2*N)=HL1p; 
 yhp(:,2*N+1:3*N)=HH1p;
 
 yhp(:,3*N+1:4*N)=LH2p; 
 yhp(:,4*N+1:5*N)=HL2p; 
 yhp(:,5*N+1:6*N)=HH2p;
 
 yhp(:,6*N+1:7*N)=LH3p; 
 yhp(:,7*N+1:8*N)=HL3p; 
 yhp(:,8*N+1:9*N)=HH3p;
 
 yhp(:,9*N+1:10*N)=LH4p; 
 yhp(:,10*N+1:11*N)=HL4p; 
 yhp(:,11*N+1:12*N)=HH4p;
 
[X,L] = mirdwt(yl,yhp,h,L);
X=X*4;
